from datetime import datetime
from traceback import print_exc

import pandas as pd
from pandas import DataFrame

from offline_results.common.config import BOTTOM_VIEWED_HOMEPAGES_LIMIT
from offline_results.common.constants import PAY_TV_CONTENT, NO_PAY_TV_CONTENT, CONTENT_ID, HAS_HOMEPAGE, HOMEPAGE_ID, \
    HOMEPAGE_STATUS, STATUS, ACTIVE, CREATED_ON, VIEW_COUNT, INNER, RECORDS, PAY_TV, IS_CONNECTED, REC_TYPE
from offline_results.utils.logger import Logging


class BottomViewedUtils:

    @staticmethod
    def get_label_wise_homepage_for_contents(graph, user_label) -> DataFrame:
        try:
            content_label = PAY_TV_CONTENT if user_label == PAY_TV else NO_PAY_TV_CONTENT
            content_response = graph.custom_query(
                query=f'''g.V().hasLabel('{content_label}').match(
                __.as("c").valueMap("{CONTENT_ID}","{STATUS}").as("{CONTENT_ID}"),
                __.as("c").out("{HAS_HOMEPAGE}")
                .valueMap("{HOMEPAGE_ID}","{HOMEPAGE_STATUS}","{IS_CONNECTED}").as("{HOMEPAGE_ID}")
                ).select("content_id", "homepage_id")''',
                payload={
                    content_label: content_label,
                    CONTENT_ID: CONTENT_ID,
                    HAS_HOMEPAGE: HAS_HOMEPAGE,
                    HOMEPAGE_ID: HOMEPAGE_ID,
                    HOMEPAGE_STATUS: HOMEPAGE_STATUS,
                    IS_CONNECTED: IS_CONNECTED,
                    STATUS: STATUS
                })
            content_homepage_map_df = pd.DataFrame()
            for query in content_response:
                for data in query:
                    content_id = data[CONTENT_ID][CONTENT_ID][0]
                    content_status = data[CONTENT_ID][STATUS][0]
                    homepage_id = data[HOMEPAGE_ID][HOMEPAGE_ID][0]
                    homepage_status = data[HOMEPAGE_ID][HOMEPAGE_STATUS][0]
                    is_connected = data[HOMEPAGE_ID][IS_CONNECTED][0]
                    content_homepage_map = pd.DataFrame([{CONTENT_ID: content_id,
                                                          STATUS: content_status,
                                                          HOMEPAGE_ID: homepage_id,
                                                          HOMEPAGE_STATUS: homepage_status,
                                                          IS_CONNECTED: is_connected}])
                    content_homepage_map_df = pd.concat([content_homepage_map_df, content_homepage_map], axis=0) \
                        .sort_values(by=CONTENT_ID).reset_index(drop=True)
            content_homepage_map_df = content_homepage_map_df[content_homepage_map_df[HOMEPAGE_STATUS] == ACTIVE] \
                .reset_index(drop=True)
        except Exception:
            print_exc()
            content_homepage_map_df = []
        return content_homepage_map_df

    @staticmethod
    def get_content_wise_view_count(df):
        try:
            content_views_df = df.groupby([CONTENT_ID], as_index=False).agg(
                view_count=(VIEW_COUNT, sum))
            content_views_df.reset_index(drop=True, inplace=True)
            return content_views_df
        except Exception as e:
            Logging.error(f"Error while calculating content-wise view-counts, Error: {e}")

    @staticmethod
    def get_homepage_wise_view_count(df):
        try:
            homepage_views_df = df.groupby([HOMEPAGE_ID], as_index=False).agg(
                view_count=(VIEW_COUNT, sum))
            return homepage_views_df
        except Exception as e:
            Logging.error(f"Error while calculating content-wise view-counts, Error: {e}")

    @staticmethod
    def add_created_on_attribute(dataframe) -> DataFrame:
        try:
            dataframe[CREATED_ON] = datetime.utcnow().isoformat()
            return dataframe
        except Exception as e:
            Logging.error(f"Error while adding time created attribute, Error: {e}")

    @staticmethod
    def add_recommendation_type_attribute(dataframe, rec_type) -> DataFrame:
        try:
            dataframe[REC_TYPE] = rec_type
            return dataframe
        except Exception as e:
            Logging.error(f"Error while adding recommendation type attribute, Error: {e}")

    @staticmethod
    def fetch_bottom_viewed_homepage_id(data, user_label):
        try:
            df = data.sort_values([VIEW_COUNT], ascending=False).reset_index(drop=True)
            df = df.tail(BOTTOM_VIEWED_HOMEPAGES_LIMIT).reset_index(drop=True)
            return df
        except Exception as e:
            Logging.error(f"Error while filtering contents with BOTTOM_VIEWED_LIMIT, Error: {e}")

    @staticmethod
    def get_inner_merged_df(data1, data2, on):
        try:
            df = pd.merge(data1, data2, on=on, how=INNER)
            return df
        except Exception as e:
            Logging.error(f"Error while merging on {on}, Error: {e}")

    @staticmethod
    def get_json_format_output(df, key_prefix):
        try:
            output_dict = {}
            df = df[[HOMEPAGE_ID, CREATED_ON, REC_TYPE]]
            output_dict[key_prefix] = df.to_dict(RECORDS)
            return output_dict
        except Exception as e:
            Logging.error(f"Error while converting df to json format for{key_prefix}, Error: {e}")
