import logging
from datetime import datetime

from pandas import merge
from sklearn.preprocessing import MinMaxScaler

from offline_results.common.constants import PAY_TV_CONTENT, PAY_TV, NO_PAY_TV_CONTENT, HOMEPAGE_ID, CONTENT_ID, \
    CONTENT_LABEL_COLUMN, INNER, VIEW_COUNT, SCORE, CREATED_ON, SERVICE_NAME, \
    RC_TOP_VIEWED_MODULE_DEFAULT, RECORDS, NO_PAY_TV, REC_TYPE
from offline_results.recommendation.utils import RecommendationUtils


class TVFallback(
    RecommendationUtils,
):

    def generate_rec(self, pay_tv_label):

        try:
            content_with_homepage = self.homepage_data(
                pay_tv_label=PAY_TV_CONTENT if pay_tv_label is PAY_TV else NO_PAY_TV_CONTENT)
            self.content_with_homepage = content_with_homepage[[HOMEPAGE_ID, CONTENT_ID]]
            self.content_with_homepage[HOMEPAGE_ID] = content_with_homepage[HOMEPAGE_ID].apply(lambda x: x[0])
            self.active_homepage_id = list(self.active_homepage_ids(is_paytv=True if pay_tv_label is PAY_TV else False))
            self.content_with_homepage = self.content_with_homepage[
                self.content_with_homepage[HOMEPAGE_ID].isin(self.active_homepage_id)]
            logging.info(" reading user viewed data from s3 .")
            data = self.user_viewed_data_from_s3()
            data = data[data[CONTENT_LABEL_COLUMN] == (PAY_TV_CONTENT if pay_tv_label is PAY_TV else NO_PAY_TV_CONTENT)]
            data = merge(data, self.content_with_homepage, on=CONTENT_ID, how=INNER)
            data[SCORE] = data[VIEW_COUNT]
            result_df = data[[CONTENT_ID, HOMEPAGE_ID, SCORE, ]]
            result_df = result_df.groupby([HOMEPAGE_ID])[SCORE].sum().reset_index()
            scaler = MinMaxScaler(feature_range=(0, 1))
            result_df[SCORE] = scaler.fit_transform(result_df[[SCORE]].to_numpy()).flatten().tolist()
            result_df = result_df.sort_values([SCORE, ], ascending=False, ignore_index=True)
            result_df[CREATED_ON] = datetime.utcnow().isoformat()
            result_df[REC_TYPE] = RC_TOP_VIEWED_MODULE_DEFAULT
            result_df = result_df[[HOMEPAGE_ID, CREATED_ON, REC_TYPE]]
            rc_top_viewed_fallback_rec = {}
            key_prefix = SERVICE_NAME + ":" + RC_TOP_VIEWED_MODULE_DEFAULT + ":" + pay_tv_label
            result = result_df.to_dict(RECORDS)
            rc_top_viewed_fallback_rec[key_prefix] = result
            return rc_top_viewed_fallback_rec
        except Exception as e:
            logging.error(
                f"Error while preparing fallback rc top viewed based recommendation for {pay_tv_label}, Error: {e}")

    @staticmethod
    def fallback_rc_top_viewed_data():

        ctl = TVFallback()

        # fallback paytv rc_top_viewed  recommendation
        pay_tv_rc_top_rec = ctl.generate_rec(PAY_TV)

        # fallback no_paytv rc_top_viewed recommendation
        no_pay_tv_rc_top_rec = ctl.generate_rec(NO_PAY_TV)
        return pay_tv_rc_top_rec, no_pay_tv_rc_top_rec
