from offline_results.common.constants import VIEW_COUNT, CREATED_ON, CONTENT_ID, \
    HOMEPAGE_ID, SERVICE_NAME, TOP_VIEWED_MODULE_NAME, PAY_TV, NO_PAY_TV, IS_PAY_TV, IS_CONNECTED, YES, NO, REC_TYPE
from offline_results.recommendation.homepage_cluster_category.rc_top_viewed.top_viewed_utils import TopViewedUtils
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.utils.logger import Logging
from offline_results.recommendation.utils import RecommendationUtils

graph = ANGraphDb.new_connection_config().graph


class TopViewed:

    @staticmethod
    def get_top_viewed_df(user_label, user_content):
        try:
            if user_label == PAY_TV:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == True]
            else:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == False]

            content_views_df = TopViewedUtils.get_content_wise_view_count(user_content_df)

            content_homepage_mapping = TopViewedUtils.get_label_wise_homepage_for_contents(graph, user_label)

            Logging.info(f"Merging log with content-homepage mapping for {user_label}")

            homepage_content_views_df = TopViewedUtils.get_inner_merged_df(content_views_df,
                                                                           content_homepage_mapping,
                                                                           on=CONTENT_ID)

            if user_label == PAY_TV:
                homepage_content_views_df = homepage_content_views_df.loc[
                    homepage_content_views_df[IS_CONNECTED] == YES]
            else:
                homepage_content_views_df = homepage_content_views_df.loc[homepage_content_views_df[IS_CONNECTED] == NO]

            homepage_views_df = TopViewedUtils.get_homepage_wise_view_count(homepage_content_views_df)

            Logging.info(f"Filtering top-viewed Homepage IDs for {user_label}")

            top_viewed_df = TopViewedUtils.fetch_top_viewed_homepage_id(homepage_views_df, user_label)

            top_viewed_df = top_viewed_df.sort_values([VIEW_COUNT], ascending=False)

            top_viewed_df = TopViewedUtils.add_created_on_attribute(top_viewed_df)
            top_viewed_df = TopViewedUtils.add_recommendation_type_attribute(top_viewed_df, TOP_VIEWED_MODULE_NAME)

            top_viewed_df = top_viewed_df[[HOMEPAGE_ID, CREATED_ON, REC_TYPE]]
            return top_viewed_df
        except Exception as e:
            Logging.error(f"Error while preparing top-viewed homepage_id rec for {user_label}, Error: {e}")

    @staticmethod
    def get_top_viewed_homepage_id():
        try:
            user_content_df = RecommendationUtils.user_viewed_data_from_s3()
            list_of_key_prefix = []
            list_label = [PAY_TV, NO_PAY_TV]
            list_output_df = []
            for user_label in list_label:
                Logging.info(f"PREPARING TOP-VIEWED HOMEPAGE_ID REC FOR {user_label}")
                output_df = TopViewed.get_top_viewed_df(user_label, user_content_df)
                key_prefix = SERVICE_NAME + ":" + TOP_VIEWED_MODULE_NAME + ":" + user_label
                list_of_key_prefix.append(key_prefix)
                list_output_df.append(output_df)
            top_viewed_pay_tv_homepage_id, top_viewed_no_pay_tv_homepage_id = list_output_df

            Logging.info("Preparing expected output schema")

            # Convert the dataframe output to json output format
            top_viewed_pay_tv_homepage_id_json = TopViewedUtils.get_json_format_output(top_viewed_pay_tv_homepage_id,
                                                                                       list_of_key_prefix[0])
            top_viewed_no_pay_tv_homepage_id_json = TopViewedUtils.get_json_format_output(
                top_viewed_no_pay_tv_homepage_id,
                list_of_key_prefix[1])
            return top_viewed_pay_tv_homepage_id_json, top_viewed_no_pay_tv_homepage_id_json
        except Exception as e:
            Logging.error(f"Error while preparing top-viewed homepage_id rec, Error: {e}")
