import logging
from pandas import concat, DataFrame
from sklearn.preprocessing import MinMaxScaler
from offline_results.common.constants import CONTENT_ID, VIEW_COUNT, CUSTOMER_ID, SCORE
from offline_results.recommendation.homepage_recommendation.core.with_cf_user_to_user.utils import UserViewershipUtils
logging.basicConfig(level=logging.INFO)


class UserViewershipController(UserViewershipUtils):

    def controller(
            self,
            content_list,
            similar_users,
            is_paytv=True,
            viewed_data=None

    ):

        #viewership = self.content_viewership(content_list=content_list, is_pay_tv=is_paytv)
        viewership = viewed_data[viewed_data[CONTENT_ID].isin(content_list)]
        viewership = viewership[viewership[CUSTOMER_ID].isin(similar_users)]
        if viewership is None:
            viewership = self.content_viewership(content_list=content_list, is_pay_tv=is_paytv)
        if viewership is None or len(viewership) == 0:
            logging.info(f"No view count available, assign default score. viewership is: {viewership}")
            return DataFrame({CONTENT_ID: content_list, SCORE: [-1] * len(content_list)})

        viewership[VIEW_COUNT] = viewership[VIEW_COUNT].fillna(0)
        max_view_count = viewership[VIEW_COUNT].max()
        filtered_viewership = viewership[viewership[CUSTOMER_ID].isin(similar_users)].copy()
        filtered_viewership[SCORE] = filtered_viewership[VIEW_COUNT] + max_view_count
        viewership = viewership.drop(index=filtered_viewership.index)
        viewership[SCORE] = viewership[VIEW_COUNT]
        viewership = concat([filtered_viewership, viewership], axis=0).reset_index(drop=True)
        content_view_counts = viewership.groupby(CONTENT_ID)[SCORE].size().reset_index()
        scaler = MinMaxScaler()
        content_view_counts[[SCORE]] = scaler.fit_transform(content_view_counts[[SCORE]])
        content_view_counts[SCORE] = 1 if content_view_counts[SCORE].max() == 0 else content_view_counts[SCORE]
        content_view_counts = content_view_counts.sort_values([SCORE], ascending=False).reset_index(drop=True)

        return content_view_counts[[CONTENT_ID, SCORE]]


