from offline_results.common.config import CONFIG_NOT_APPLICABLE_FOR_REC_PAYTV, CONFIG_NOT_APPLICABLE_FOR_REC_NO_PAYTV
from offline_results.common.constants import CREATED_ON, CONTENT_ID, HOMEPAGE_ID, IS_PAY_TV, PAY_TV, NO_PAY_TV, \
    SERVICE_NAME, ALL_CONTENT_BASED, HOMEPAGE_ID_BASED, VIEW_COUNT, FALLBACK_LEAST_VIEWED_MODULE_NAME, REC_TYPE
from offline_results.recommendation.homepage_recommendation.core.with_least_viewed.lv_utils import LVUtils
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.utils.logger import Logging

graph = ANGraphDb.new_connection_config().graph


class LVFallback:

    @staticmethod
    def get_fallback_all_contents_based_df(user_label, homepage_id_wise, user_content):
        try:
            if user_label == PAY_TV:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == True]
            else:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == False]

            active_contents_df = LVUtils.get_label_wise_active_contents(graph, user_label)

            Logging.info(f"Merging log with active contents for {user_label}")

            least_viewed_df = LVUtils.get_inner_merged_df(user_content_df,
                                                          active_contents_df,
                                                          on=CONTENT_ID)

            least_viewed_df = LVUtils.get_default_least_viewed_contents(user_content_df)

            least_viewed_df = least_viewed_df.drop_duplicates(subset=[CONTENT_ID]).reset_index(drop=True)

            least_viewed_df = LVUtils.add_created_on_attribute(least_viewed_df)
            least_viewed_df = LVUtils.add_recommendation_type_attribute(least_viewed_df,
                                                                        FALLBACK_LEAST_VIEWED_MODULE_NAME)

            least_viewed_df = least_viewed_df.sort_values([VIEW_COUNT], ascending=False).reset_index(drop=True)
            least_viewed_df = least_viewed_df[
                [CONTENT_ID, CREATED_ON, REC_TYPE]].reset_index(drop=True)
            return least_viewed_df
        except Exception as e:
            Logging.error(f"Error while preparing FALLBACK least viewed rec for {user_label}, Error: {e}")

    @staticmethod
    def get_fallback_homepage_id_wise_df(user_label, homepage_id_wise, user_content):
        try:
            if user_label == PAY_TV:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == True]
                homepage_not_applicable_for_rec = list(CONFIG_NOT_APPLICABLE_FOR_REC_PAYTV.keys())
            else:
                user_content_df = user_content.loc[user_content[IS_PAY_TV] == False]
                homepage_not_applicable_for_rec = list(CONFIG_NOT_APPLICABLE_FOR_REC_NO_PAYTV.keys())

            content_homepage_mapping = LVUtils.get_label_wise_homepage_for_contents(graph, user_label, homepage_id_wise)

            Logging.info(f"Merging log with content-homepage mapping for {user_label}")

            homepage_mapped_user_content_df = LVUtils.get_inner_merged_df(user_content_df,
                                                                          content_homepage_mapping,
                                                                          on=CONTENT_ID)

            homepage_mapped_user_content_df = homepage_mapped_user_content_df.explode(column=HOMEPAGE_ID)
            homepage_mapped_user_content_df = homepage_mapped_user_content_df[
                ~homepage_mapped_user_content_df[HOMEPAGE_ID].isnull()].reset_index(drop=True)

            homepage_mapped_user_content_df = homepage_mapped_user_content_df[
                ~homepage_mapped_user_content_df[HOMEPAGE_ID].isin(homepage_not_applicable_for_rec)]. \
                reset_index(drop=True)

            least_viewed_df = LVUtils.get_default_homepage_wise_least_viewed_contents(
                homepage_mapped_user_content_df)

            least_viewed_df = least_viewed_df.drop_duplicates(subset=[HOMEPAGE_ID, CONTENT_ID]).reset_index(drop=True)

            least_viewed_df = LVUtils.add_created_on_attribute(least_viewed_df)
            least_viewed_df = LVUtils.add_recommendation_type_attribute(least_viewed_df,
                                                                        FALLBACK_LEAST_VIEWED_MODULE_NAME)

            least_viewed_df = least_viewed_df.sort_values([HOMEPAGE_ID, VIEW_COUNT],
                                                          ascending=(True, False)). \
                reset_index(drop=True)
            least_viewed_df = least_viewed_df[[HOMEPAGE_ID, CONTENT_ID, CREATED_ON, REC_TYPE]]. \
                reset_index(drop=True)
            return least_viewed_df
        except Exception as e:
            Logging.error(f"Error while preparing FALLBACK least viewed rec for {user_label}, Error: {e}")

    @staticmethod
    def get_fallback_least_viewed_all_contents():
        try:
            homepage_id_wise = False

            user_content_df = RecommendationUtils.user_viewed_data_from_s3()

            list_of_key_prefix = []
            list_label = [PAY_TV, NO_PAY_TV]
            list_output_df = []
            for user_label in list_label:
                Logging.info(f"PREPARING FALLBACK least VIEWED REC BASED ON ALL CONTENTS FOR {user_label}")
                output_df = LVFallback.get_fallback_all_contents_based_df(user_label, homepage_id_wise,
                                                                          user_content_df)
                key_prefix = SERVICE_NAME + ":" + FALLBACK_LEAST_VIEWED_MODULE_NAME + ":" + user_label + ":" + ALL_CONTENT_BASED
                list_of_key_prefix.append(key_prefix)
                list_output_df.append(output_df)
            least_viewed_pay_tv_all_content, least_viewed_no_pay_tv_all_content = list_output_df

            Logging.info("Preparing FALLBACK expected output schema")

            # Convert the dataframe output to json output format
            least_viewed_pay_tv_all_content_json = LVUtils.get_default_json_format_output(
                least_viewed_pay_tv_all_content,
                list_of_key_prefix[0],
                homepage_id_wise)
            least_viewed_no_pay_tv_all_content_json = LVUtils.get_default_json_format_output(
                least_viewed_no_pay_tv_all_content,
                list_of_key_prefix[1],
                homepage_id_wise)
            return least_viewed_pay_tv_all_content_json, least_viewed_no_pay_tv_all_content_json
        except Exception as e:
            Logging.error(f"Error while preparing FALLBACK all_content_based rec, Error: {e}")

    @staticmethod
    def get_fallback_least_viewed_homepage_id():
        try:
            homepage_id_wise = True

            user_content_df = RecommendationUtils.user_viewed_data_from_s3()

            list_of_key_prefix = []
            list_label = [PAY_TV, NO_PAY_TV]
            list_output_df = []
            for user_label in list_label:
                Logging.info(f"PREPARING FALLBACK least VIEWED REC BASED ON HOMEPAGE_ID FOR {user_label}")
                output_df = LVFallback.get_fallback_homepage_id_wise_df(user_label, homepage_id_wise,
                                                                        user_content_df)
                key_prefix = SERVICE_NAME + ":" + FALLBACK_LEAST_VIEWED_MODULE_NAME + ":" + user_label + ":" + HOMEPAGE_ID_BASED
                list_of_key_prefix.append(key_prefix)
                list_output_df.append(output_df)
            least_viewed_pay_tv_homepage_id, least_viewed_no_pay_tv_homepage_id = list_output_df

            Logging.info("Preparing FALLBACK expected output schema")

            # Convert the dataframe output to json output format
            least_viewed_pay_tv_homepage_id_json = LVUtils.get_default_json_format_output(
                least_viewed_pay_tv_homepage_id,
                list_of_key_prefix[0],
                homepage_id_wise)
            least_viewed_no_pay_tv_homepage_id_json = LVUtils.get_default_json_format_output(
                least_viewed_no_pay_tv_homepage_id,
                list_of_key_prefix[1],
                homepage_id_wise)
            return least_viewed_pay_tv_homepage_id_json, least_viewed_no_pay_tv_homepage_id_json
        except Exception as e:
            Logging.error(f"Error while preparing FALLBACK homepage_id_based rec, Error: {e}")
