import time
from datetime import datetime
from pandas import DataFrame
from offline_results.common.config import CONFIG_HOMEPAGE_PAYTV, CONFIG_HOMEPAGE_NO_PAYTV
from offline_results.common.constants import PAY_TV_CONTENT, NO_PAY_TV_CONTENT, TV_CHANNEL_LIST, CONTENT_ID, \
    HOMEPAGE_ID, MODEL_POPULAR, POPULAR, SERVICE_NAME, ALL_CONTENT_BASED, POPULAR_MODULE_DEFAULT, RECORDS, \
    PAY_TV, LEFT, ACTIVE, CONTENT_STATUS, IS_PAY_TV, CREATED_ON, VIEW_COUNT, SUM, \
    HOMEPAGE_ID_BASED, REC_TYPE, POPULAR_FALLBACK, NO_PAY_TV
from offline_results.recommendation.homepage_recommendation.trending.trending_model import TrendingModel
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.similarity.content_profile.query_utils import QueryUtils
from offline_results.utils.logger import Logging


class PopularModelFallback:

    viewed_relation_history_df = DataFrame()

    @staticmethod
    def get_data():
        obj = PopularModelFallback
        if len(obj.viewed_relation_history_df) != 0: return
        obj.viewed_relation_history_df = RecommendationUtils.user_viewed_data_from_s3()

    @staticmethod
    def clear_data():
        obj = PopularModelFallback
        obj.viewed_relation_history_df = DataFrame()

    @staticmethod
    def get_homepage_for_contents(
            is_pay_tv
    ) -> DataFrame:
        try:
            query = f'''g.V().hasLabel('{PAY_TV_CONTENT if is_pay_tv else NO_PAY_TV_CONTENT}' ).match(
                __.as("c").values().as("content_id"),
                __.as("c").out("HAS_HOMEPAGE")
                .values("homepage_id").as("homepage_id")
                ).select("content_id", "homepage_id")'''
            graph = ANGraphDb.new_connection_config().graph
            data = graph.custom_query(query, payload={PAY_TV_CONTENT: PAY_TV_CONTENT,
                                                      NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT})
            data = [rec for idx in data for rec in idx]
            df = DataFrame(data)
            return df
        except Exception:
            graph = ANGraphDb.new_connection_config().graph
            query = f'''g.V().hasLabel('{PAY_TV_CONTENT if is_pay_tv else NO_PAY_TV_CONTENT}' ).match(
                            __.as("c").values().as("content_id"),
                            __.as("c").out("HAS_HOMEPAGE")
                            .values("homepage_id").as("homepage_id")
                            ).select("content_id", "homepage_id")'''

            data = graph.custom_query(query, payload={PAY_TV_CONTENT: PAY_TV_CONTENT,
                                                      NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT})
            data = [rec for idx in data for rec in idx]
            df = DataFrame(data)
            return df

    @staticmethod
    def default_popular_all_content_algorithm(pay_tv_label):
        df = PopularModelFallback.viewed_relation_history_df
        label = True if pay_tv_label == PAY_TV else False
        df_pay_tv = df[(df[IS_PAY_TV] == label) & (df[CONTENT_STATUS] == ACTIVE)]
        try:
            graph = ANGraphDb.new_connection_config().graph
            Logging.info("Fetch content-homepage_id mapping")
            df_content_homepage = TrendingModel.get_content_homepage_id_mapping(graph, pay_tv_label,
                                                                                homepage_id_wise=False)
        except Exception:
            graph = ANGraphDb.new_connection_config().graph
            Logging.info("Re-trying to Fetch content-homepage_id mapping")
            df_content_homepage = TrendingModel.get_content_homepage_id_mapping(graph, pay_tv_label,
                                                                                homepage_id_wise=False)
        df_pay_tv = df_pay_tv.merge(df_content_homepage, on=CONTENT_ID, how=LEFT)
        df_pay_tv = df_pay_tv[~df_pay_tv[HOMEPAGE_ID].isna()].reset_index(drop=True)
        df_pay_tv = df_pay_tv.explode(column=HOMEPAGE_ID)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if pay_tv_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        df_sum = df_pay_tv.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
        df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False).iloc[:20, :]
        df_sum = df_sum.merge(PopularModelFallback.get_homepage_for_contents(label),
                              on=CONTENT_ID, how=LEFT)
        df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
        df_sum.drop_duplicates(CONTENT_ID, inplace=True)
        df_sum[CREATED_ON] = datetime.utcnow().isoformat()
        df_sum[REC_TYPE] = POPULAR_FALLBACK
        all_content_based_rec = {}
        key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + pay_tv_label + ":" + ALL_CONTENT_BASED
        result = df_sum.to_dict(orient=RECORDS)
        all_content_based_rec[key_prefix] = result
        return all_content_based_rec

    @staticmethod
    def default_popular_homepage_id_algorithm(user_label):
        df = PopularModelFallback.viewed_relation_history_df
        label = True if user_label == PAY_TV else False
        df_pay_tv = df[(df[IS_PAY_TV] == label) & (df[CONTENT_STATUS] == ACTIVE)]
        try:
            graph = ANGraphDb.new_connection_config().graph
            Logging.info("Fetch content-homepage_id mapping")
            df_active_homepage = TrendingModel.get_content_homepage_id_mapping(graph, user_label,
                                                                               homepage_id_wise=True)
        except Exception:
            graph = ANGraphDb.new_connection_config().graph
            Logging.info("Re-trying to Fetch content-homepage_id mapping")
            df_active_homepage = TrendingModel.get_content_homepage_id_mapping(graph, user_label,
                                                                               homepage_id_wise=True)
        df_pay_tv = df_pay_tv.merge(df_active_homepage, on=CONTENT_ID, how=LEFT)
        df_pay_tv = df_pay_tv[~df_pay_tv[HOMEPAGE_ID].isna()].reset_index(drop=True)
        df_pay_tv = df_pay_tv.explode(column=HOMEPAGE_ID)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if user_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        unique_homepage_id = df_pay_tv[HOMEPAGE_ID].unique()
        homepage_based_rec = {}
        for homepage_id in unique_homepage_id:
            key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + user_label + ":" + HOMEPAGE_ID_BASED + ":" + str(
                homepage_id)
            df_homepage = df_pay_tv[df_pay_tv[HOMEPAGE_ID] == homepage_id]
            df_sum = df_homepage.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
            df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False)
            df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
            df_sum.drop_duplicates(CONTENT_ID, inplace=True)
            df_sum[CREATED_ON] = datetime.utcnow().isoformat()
            df_sum[REC_TYPE] = POPULAR_FALLBACK
            result = df_sum.to_dict(orient=RECORDS)
            homepage_based_rec[key_prefix] = result
        return homepage_based_rec

    @staticmethod
    def get_default_popular_homepage_id():
        PopularModelFallback.clear_data()
        PopularModelFallback.get_data()
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_dict = []
        Logging.info("Get Popular Fallback Content based on Homepage_id")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating fallback popular content based on Homepage_id for " + user_label + " users")
            output_dict = PopularModelFallback.default_popular_homepage_id_algorithm(user_label)
            Logging.info("Success Preparing Fallback Trending Dataframe Output")
            list_output_dict.append(output_dict)
        fallback_popular_pay_tv_homepage_id_dict, fallback_popular_no_pay_tv_homepage_id_dict = list_output_dict
        end = time.time()
        duration = end-start
        Logging.info("Duration for Get Fallback Popular Content based on Homepage_id is " + str(duration) + " seconds")
        return fallback_popular_pay_tv_homepage_id_dict, fallback_popular_no_pay_tv_homepage_id_dict

    @staticmethod
    def get_default_popular_all_content():
        PopularModelFallback.get_data()
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_dict = []
        Logging.info("Get Fallback Popular Content based on All Content")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating fallback popular content based on All Content for " + user_label + " users")
            output_dict = PopularModelFallback.default_popular_all_content_algorithm(user_label)
            Logging.info("Success Preparing Fallback Popular Dataframe Output")
            list_output_dict.append(output_dict)
        fallback_popular_pay_tv_all_content_dict, fallback_popular_no_pay_tv_all_content_dict = list_output_dict
        end = time.time()
        duration = end - start
        Logging.info("Duration for Get Fallback Popular Content based on All Content is " + str(duration) + " seconds")
        PopularModelFallback.clear_data()
        return fallback_popular_pay_tv_all_content_dict, fallback_popular_no_pay_tv_all_content_dict
