import ast
import time
import pandas as pd
from sklearn.preprocessing import MinMaxScaler
from datetime import datetime
from hijri_converter import Hijri, Gregorian

from offline_results.common.config import CONFIG_HOMEPAGE_PAYTV, CONFIG_HOMEPAGE_NO_PAYTV, VISIONPLUS_DEV
from offline_results.common.constants import CONTENT_ID, VIEW_COUNT, DURATION, UBD_DAYOFWEEK, SCALED_DURATION, \
    SCALED_VIEW_COUNT, INNER, UBD_DAYOFYEAR, UBD_CREATED_ON, SEASON, WEEKEND, WEEKDAY, NEW_YEAR, CHRISTMAS, VALENTINE, \
    SCORE, CREATED_ON, LEFT, HOMEPAGE_ID, SERVICE_NAME, ALL_CONTENT_BASED, SEASONAL_MODULE_NAME, RECORDS, \
    HOMEPAGE_ID_BASED, ACCESS_DATE, ACCESS_DATE_DAYOFYEAR, ACCESS_DATE_DAYOFWEEK, YES, NO, HOMEPAGE, IS_CONNECTED, \
    HOMEPAGE_TITLE_EN, CHRISTMAS_RELEVANT_PAYTV_HOMEPAGE, NEW_YEAR_RELEVANT_PAYTV_HOMEPAGE, WEEKDAY_SEASON_PERIOD, \
    VALENTINE_RELEVANT_PAYTV_HOMEPAGE, WEEKEND_RELEVANT_HOMEPAGE, WEEKDAY_RELEVANT_HOMEPAGE, \
    CHRISTMAS_RELEVANT_NOPAYTV_HOMEPAGE_ID, NEW_YEAR_RELEVANT_NOPAYTV_HOMEPAGE_ID, VALENTINE_RELEVANT_NOPAYTV_HOMEPAGE, \
    NEW_YEAR_SEASON_PERIOD, CHRISTMAS_SEASON_PERIOD, VALENTINE_SEASON_PERIOD, WEEKEND_SEASON_PERIOD, PAY_TV, NO_PAY_TV, \
    RAMADAN_SEASON_PERIOD, PREVIOUS_RAMADAN_SEASON_PERIOD, RAMADAN, RAMADAN_RELEVANT_PAYTV_HOMEPAGE, \
    RAMADAN_RELEVANT_NOPAYTV_HOMEPAGE, DEFAULT_PERIOD, REC_TYPE, SEASONAL, UBD_WEEKOFYEAR, TV_CHANNEL_LIST, IS_PAY_TV, \
    MODEL_NAME, VIEW_HISTORY

from offline_results.recommendation.homepage_recommendation.trending.trending_model import TrendingModel
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.utils.logger import Logging


class SeasonalModel:

    viewed_relation_history_df = pd.DataFrame()

    @staticmethod
    def get_data():
        obj = SeasonalModel
        if len(obj.viewed_relation_history_df) != 0: return
        obj.viewed_relation_history_df = RecommendationUtils.user_viewed_data_from_s3()

    @staticmethod
    def clear_data():
        obj = SeasonalModel
        obj.viewed_relation_history_df = pd.DataFrame()

    @staticmethod
    def get_dict_format_output(df, key_prefix, homepage_id_wise):
        if homepage_id_wise:
            output_dict = {}
            unique_homepage_id = df[HOMEPAGE_ID].unique()
            for homepage_id in unique_homepage_id:
                key_prefix_cls_hid = key_prefix + ":" + str(homepage_id)
                homepage_wise_df = df.loc[df[HOMEPAGE_ID] == homepage_id]
                homepage_wise_df = homepage_wise_df[[CONTENT_ID, SCORE, CREATED_ON, REC_TYPE, SEASON]]
                output_dict[key_prefix_cls_hid] = homepage_wise_df.to_dict(RECORDS)
        else:
            df = df[[CONTENT_ID, HOMEPAGE_ID, SCORE, CREATED_ON, REC_TYPE, SEASON]]
            output_dict = {key_prefix: df.to_dict(RECORDS)}
        return output_dict

    @staticmethod
    def ramadan_date_validator(access_date):
        gregorian_year = access_date.year
        gregorian_month = access_date.month
        gregorian_day = access_date.day
        hijri_date = Gregorian(gregorian_year, gregorian_month, gregorian_day).to_hijri()
        hijri_date_month = hijri_date.month
        hijri_date_day = hijri_date.day
        ramadan_status = True if hijri_date_day in [1, 2, 3] and hijri_date_month in [10] else False

        return ramadan_status, hijri_date

    @staticmethod
    def generate_access_date_detail(access_date, ramadan_status, hijri_date):
        if ramadan_status:
            access_date_df = pd.DataFrame([{ACCESS_DATE: access_date}])
            access_date_df[ACCESS_DATE_DAYOFYEAR] = access_date_df[ACCESS_DATE].dt.dayofyear
            access_date_df[ACCESS_DATE_DAYOFWEEK] = access_date_df[ACCESS_DATE].dt.dayofweek
            access_date_dayofyear = access_date_df[ACCESS_DATE_DAYOFYEAR][0]
            access_date_dayofweek = access_date_df[ACCESS_DATE_DAYOFWEEK][0]
            is_weekend = access_date_dayofweek > 4
            access_date_dayofyear_ramadan_theday = access_date_dayofyear
            access_date_dayofyear_ramadan_dayminusone = access_date_dayofyear - 1
            access_date_dayofyear_ramadan_dayplusone = access_date_dayofyear + 1
            RAMADAN_SEASON_PERIOD.append(access_date_dayofyear_ramadan_dayminusone)
            RAMADAN_SEASON_PERIOD.append(access_date_dayofyear_ramadan_theday)
            RAMADAN_SEASON_PERIOD.append(access_date_dayofyear_ramadan_dayplusone)

            previous_ramadan_date_year = hijri_date.year - 1
            previous_ramadan_date_month = hijri_date.month
            previous_ramadan_date_day = hijri_date.day
            previous_ramadan_date_gregorian_calendar = Hijri(previous_ramadan_date_year, previous_ramadan_date_month,
                                                             previous_ramadan_date_day).to_gregorian()
            previous_ramadan_date_gregorian_calendar = pd.Series(datetime(
                year=previous_ramadan_date_gregorian_calendar.year, month=previous_ramadan_date_gregorian_calendar.month
                , day=previous_ramadan_date_gregorian_calendar.day))
            previous_ramadan_date_theday = previous_ramadan_date_gregorian_calendar.dt.dayofyear[0]
            previous_ramadan_date_dayminusone = previous_ramadan_date_theday - 1
            previous_ramadan_date_dayplusone = previous_ramadan_date_theday + 1
            PREVIOUS_RAMADAN_SEASON_PERIOD.append(previous_ramadan_date_dayminusone)
            PREVIOUS_RAMADAN_SEASON_PERIOD.append(previous_ramadan_date_theday)
            PREVIOUS_RAMADAN_SEASON_PERIOD.append(previous_ramadan_date_dayplusone)
            return access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD, PREVIOUS_RAMADAN_SEASON_PERIOD
        else:
            access_date_df = pd.DataFrame([{ACCESS_DATE: access_date}])
            access_date_df[ACCESS_DATE_DAYOFYEAR] = access_date_df[ACCESS_DATE].dt.dayofyear
            access_date_df[ACCESS_DATE_DAYOFWEEK] = access_date_df[ACCESS_DATE].dt.dayofweek
            access_date_dayofyear = access_date_df[ACCESS_DATE_DAYOFYEAR][0]
            access_date_dayofweek = access_date_df[ACCESS_DATE_DAYOFWEEK][0]
            is_weekend = access_date_dayofweek > 4

            return access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD, PREVIOUS_RAMADAN_SEASON_PERIOD

    @staticmethod
    def get_access_date_condition(history_content_df):
        Logging.info("Fetch user access date of today")
        history_content_df[UBD_DAYOFYEAR] = history_content_df[UBD_CREATED_ON].dt.dayofyear
        # Example to create custom access date (FOR TESTING PURPOSE)
        # custom_access_date = "2022-12-25 10:10:10"
        # access_date = datetime.fromisoformat(custom_access_date)

        access_date = datetime.today()
        Logging.info("User access date of today : " + str(access_date))
        ramadan_status, hijri_date = SeasonalModel.ramadan_date_validator(access_date)
        return SeasonalModel.generate_access_date_detail(access_date, ramadan_status, hijri_date)

    @staticmethod
    def specific_season_generator(history_content_df, access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD,
                                  PREVIOUS_RAMADAN_SEASON_PERIOD):
        Logging.info("Generating specific season dataframe based on user access date of today")
        if is_weekend and access_date_dayofyear in NEW_YEAR_SEASON_PERIOD:
            Logging.info("User accessing the model in the weekend and new year season")
            current_season_name = NEW_YEAR
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(NEW_YEAR_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif access_date_dayofyear in NEW_YEAR_SEASON_PERIOD:
            Logging.info("User accessing the model in the new year season")
            current_season_name = NEW_YEAR
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(NEW_YEAR_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in CHRISTMAS_SEASON_PERIOD:
            Logging.info("User accessing the model in the weekend and christmas season")
            current_season_name = CHRISTMAS
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(CHRISTMAS_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif access_date_dayofyear in CHRISTMAS_SEASON_PERIOD:
            Logging.info("User accessing the model in christmas season")
            current_season_name = CHRISTMAS
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(CHRISTMAS_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in VALENTINE_SEASON_PERIOD:
            Logging.info("User accessing the model in weekend and valentine season")
            current_season_name = VALENTINE
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(VALENTINE_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif access_date_dayofyear in VALENTINE_SEASON_PERIOD:
            Logging.info("User accessing the model in valentine season")
            current_season_name = VALENTINE
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(VALENTINE_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in RAMADAN_SEASON_PERIOD:
            Logging.info("User accessing the model in weekend and ramadan season")
            current_season_name = RAMADAN
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(PREVIOUS_RAMADAN_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif access_date_dayofyear in RAMADAN_SEASON_PERIOD:
            Logging.info("User accessing the model in ramadan season")
            current_season_name = RAMADAN
            season_df = history_content_df[history_content_df[UBD_DAYOFYEAR].isin(PREVIOUS_RAMADAN_SEASON_PERIOD)] \
                .reset_index(drop=True)
        elif is_weekend:
            Logging.info("User accessing the model in the weekend")
            current_season_name = WEEKEND
            season_df = history_content_df[history_content_df[UBD_DAYOFWEEK].isin(WEEKEND_SEASON_PERIOD)] \
                .reset_index(drop=True)
        else:
            Logging.info("User accessing the model in the weekday")
            current_season_name = WEEKDAY
            season_df = history_content_df[history_content_df[UBD_DAYOFWEEK].isin(WEEKDAY_SEASON_PERIOD)] \
                .reset_index(drop=True)
        return season_df, current_season_name

    @staticmethod
    def specific_season_generator_default(history_content_df, access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD):
        Logging.info("Generating specific season dataframe based on user access date of today")
        if is_weekend and access_date_dayofyear in NEW_YEAR_SEASON_PERIOD:
            Logging.info("User accessing the model in the weekend and new year season")
            current_season_name = NEW_YEAR
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif access_date_dayofyear in NEW_YEAR_SEASON_PERIOD:
            Logging.info("User accessing the model in the new year season")
            current_season_name = NEW_YEAR
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in CHRISTMAS_SEASON_PERIOD:
            Logging.info("User accessing the model in the weekend and christmas season")
            current_season_name = CHRISTMAS
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif access_date_dayofyear in CHRISTMAS_SEASON_PERIOD:
            Logging.info("User accessing the model in christmas season")
            current_season_name = CHRISTMAS
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in VALENTINE_SEASON_PERIOD:
            Logging.info("User accessing the model in weekend and valentine season")
            current_season_name = VALENTINE
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif access_date_dayofyear in VALENTINE_SEASON_PERIOD:
            Logging.info("User accessing the model in valentine season")
            current_season_name = VALENTINE
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif is_weekend and access_date_dayofyear in RAMADAN_SEASON_PERIOD:
            Logging.info("User accessing the model in weekend and ramadan season")
            current_season_name = RAMADAN
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif access_date_dayofyear in RAMADAN_SEASON_PERIOD:
            Logging.info("User accessing the model in ramadan season")
            current_season_name = RAMADAN
            season_df = history_content_df[~history_content_df[UBD_DAYOFYEAR].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        elif is_weekend:
            Logging.info("User accessing the model in the weekend")
            current_season_name = WEEKEND
            season_df = history_content_df[~history_content_df[UBD_DAYOFWEEK].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        else:
            Logging.info("User accessing the model in the weekday")
            current_season_name = WEEKDAY
            season_df = history_content_df[~history_content_df[UBD_DAYOFWEEK].isin(DEFAULT_PERIOD)].reset_index(drop=True)
        return season_df, current_season_name

    @staticmethod
    def generate_specific_season(history_content_df, access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD,
                                 PREVIOUS_RAMADAN_SEASON_PERIOD, default_status):
        if default_status:
            season_df, current_season_name = SeasonalModel.specific_season_generator_default(history_content_df,
                                                                                             access_date_dayofyear,
                                                                                             is_weekend,
                                                                                             RAMADAN_SEASON_PERIOD)
        else:
            season_df, current_season_name = SeasonalModel.specific_season_generator(history_content_df,
                                                                                     access_date_dayofyear,
                                                                                     is_weekend,
                                                                                     RAMADAN_SEASON_PERIOD,
                                                                                     PREVIOUS_RAMADAN_SEASON_PERIOD)

        return season_df, current_season_name

    @staticmethod
    def get_specific_seasons(history_content_df, default_status=False):
        access_date_dayofyear, is_weekend, RAMADAN_SEASON_PERIOD, PREVIOUS_RAMADAN_SEASON_PERIOD = SeasonalModel\
            .get_access_date_condition(history_content_df)
        season_df, current_season_name = SeasonalModel.generate_specific_season(history_content_df,
                                                                                access_date_dayofyear,
                                                                                is_weekend, RAMADAN_SEASON_PERIOD,
                                                                                PREVIOUS_RAMADAN_SEASON_PERIOD,
                                                                                default_status)
        return season_df, current_season_name

    @staticmethod
    def get_homepage_id_detail(graph, user_label):
        is_connected_value = YES if user_label == PAY_TV else NO
        Logging.info("Fetch Homepage_id Detail for " + user_label + "_content")
        homepage_queries = graph.custom_query(f'''
        g.V().hasLabel('{HOMEPAGE}').has('{IS_CONNECTED}','{is_connected_value}').valueMap()
        ''', payload={HOMEPAGE: HOMEPAGE,
                      IS_CONNECTED: IS_CONNECTED,
                      is_connected_value: is_connected_value})
        homepage_id_detail_df = pd.DataFrame()
        for list_homepage_query in homepage_queries:
            for homepage_data in list_homepage_query:
                homepage_id = homepage_data[HOMEPAGE_ID][0]
                homepage_title_en = homepage_data[HOMEPAGE_TITLE_EN][0]
                homepage_data_df = pd.DataFrame([{HOMEPAGE_ID: homepage_id,
                                                  HOMEPAGE_TITLE_EN: homepage_title_en}])
                homepage_id_detail_df = pd.concat([homepage_id_detail_df, homepage_data_df], axis=0) \
                    .sort_values(by=HOMEPAGE_ID).reset_index(drop=True)
        homepage_id_detail_df = homepage_id_detail_df.sort_values(by=HOMEPAGE_TITLE_EN)
        return homepage_id_detail_df

    @staticmethod
    def filter_by_relevant_homepage_id(seasonal_content_df, current_season_name, user_label):
        Logging.info("Get Relevant Homepage_id Content Based on " + current_season_name + " season")
        if user_label == PAY_TV:
            season_homepage_dictionary_pay_tv = {CHRISTMAS: CHRISTMAS_RELEVANT_PAYTV_HOMEPAGE,
                                                 NEW_YEAR: NEW_YEAR_RELEVANT_PAYTV_HOMEPAGE,
                                                 VALENTINE: VALENTINE_RELEVANT_PAYTV_HOMEPAGE,
                                                 RAMADAN: RAMADAN_RELEVANT_PAYTV_HOMEPAGE,
                                                 WEEKEND: WEEKEND_RELEVANT_HOMEPAGE,
                                                 WEEKDAY: WEEKDAY_RELEVANT_HOMEPAGE}
            if current_season_name in [WEEKDAY, WEEKEND]:
                seasonal_content_df = seasonal_content_df[
                    ~seasonal_content_df[HOMEPAGE_ID].isin(season_homepage_dictionary_pay_tv[current_season_name])
                ].reset_index(drop=True)
            else:
                seasonal_content_df = seasonal_content_df[
                    seasonal_content_df[HOMEPAGE_ID].isin(season_homepage_dictionary_pay_tv[current_season_name])
                ].reset_index(drop=True)
        else:
            season_homepage_dictionary_no_pay_tv = {CHRISTMAS: CHRISTMAS_RELEVANT_NOPAYTV_HOMEPAGE_ID,
                                                    NEW_YEAR: NEW_YEAR_RELEVANT_NOPAYTV_HOMEPAGE_ID,
                                                    VALENTINE: VALENTINE_RELEVANT_NOPAYTV_HOMEPAGE,
                                                    RAMADAN: RAMADAN_RELEVANT_NOPAYTV_HOMEPAGE,
                                                    WEEKEND: WEEKEND_RELEVANT_HOMEPAGE,
                                                    WEEKDAY: WEEKDAY_RELEVANT_HOMEPAGE}
            if current_season_name in [WEEKDAY, WEEKEND]:
                seasonal_content_df = seasonal_content_df[
                    ~seasonal_content_df[HOMEPAGE_ID].isin(season_homepage_dictionary_no_pay_tv[current_season_name])
                ].reset_index(drop=True)
            else:
                seasonal_content_df = seasonal_content_df[
                    seasonal_content_df[HOMEPAGE_ID].isin(season_homepage_dictionary_no_pay_tv[current_season_name])
                ].reset_index(drop=True)
        return seasonal_content_df

    @staticmethod
    def filter_by_dedicated_homepage_id(df, user_label, filter_on_ubd):
        config_dictionary = CONFIG_HOMEPAGE_PAYTV if user_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV
        config_df = pd.DataFrame(config_dictionary.items())
        config_df.columns = [HOMEPAGE_ID, MODEL_NAME]
        config_df = config_df[config_df[MODEL_NAME] == SEASONAL]
        list_dedicated_homepage_id = config_df[HOMEPAGE_ID].tolist()
        df = df.explode(column=HOMEPAGE_ID)
        df = df[df[HOMEPAGE_ID].isin(list_dedicated_homepage_id)]\
            .reset_index(drop=True)
        if filter_on_ubd:
            history_content_df = df.drop_duplicates([CONTENT_ID, UBD_CREATED_ON])
        else:
            seasonal_content_df = df.drop_duplicates([CONTENT_ID, CREATED_ON])
        return history_content_df if filter_on_ubd else seasonal_content_df

    @staticmethod
    def fetch_historical_data(graph, user_label, homepage_id_wise):
        try:
            Logging.info("Fetch content-homepage_id mapping")
            content_homepage_id_mapping = TrendingModel.get_content_homepage_id_mapping(graph, user_label,
                                                                                        homepage_id_wise)
        except Exception:
            graph = ANGraphDb.new_connection_config().graph
            Logging.info("Re-trying to Fetch content-homepage_id mapping")
            content_homepage_id_mapping = TrendingModel.get_content_homepage_id_mapping(graph, user_label,
                                                                                        homepage_id_wise)
        Logging.info("Load Viewed Relation Network from S3")
        viewed_relation_history_df = SeasonalModel.viewed_relation_history_df
        is_pay_tv_status = True if user_label == PAY_TV else False
        viewed_relation_history_df = viewed_relation_history_df[
            viewed_relation_history_df[IS_PAY_TV] == is_pay_tv_status]
        viewed_relation_history_df = viewed_relation_history_df[
            ~viewed_relation_history_df[CONTENT_ID].isin(TV_CHANNEL_LIST)].reset_index(drop=True)
        Logging.info("Generating UBD Dataframe for " + user_label + " users")
        view_history = viewed_relation_history_df[VIEW_HISTORY].to_list()
        view_history2 = [ast.literal_eval(i) for i in view_history]
        history_content_df = pd.DataFrame({CONTENT_ID: viewed_relation_history_df[CONTENT_ID].to_list(),
                                           VIEW_HISTORY: view_history2})
        history_content_df = history_content_df.explode(column=VIEW_HISTORY)
        history_content_df[DURATION] = history_content_df[VIEW_HISTORY].apply(lambda x: x[DURATION])
        history_content_df[UBD_CREATED_ON] = history_content_df[VIEW_HISTORY].apply(lambda x: x[CREATED_ON])
        history_content_df[UBD_CREATED_ON] = pd.to_datetime(history_content_df[UBD_CREATED_ON], dayfirst=True)
        history_content_df = history_content_df.sort_values(by=UBD_CREATED_ON, ascending=False).reset_index(drop=True)
        history_content_df[UBD_WEEKOFYEAR] = history_content_df[UBD_CREATED_ON].dt.isocalendar().week
        history_content_df[UBD_DAYOFWEEK] = history_content_df[UBD_CREATED_ON].dt.dayofweek
        history_content_df = pd.merge(history_content_df, content_homepage_id_mapping, on=CONTENT_ID, how=LEFT)
        Logging.info("Filter by dedicated homepage_id for Seasonal Model")
        history_content_df = SeasonalModel.filter_by_dedicated_homepage_id(history_content_df, user_label,
                                                                           filter_on_ubd=True)
        return history_content_df, content_homepage_id_mapping

    @staticmethod
    def seasonal_algorithm(graph, user_label, homepage_id_wise):
        history_content_df, content_homepage_id_mapping = SeasonalModel.fetch_historical_data(graph, user_label,
                                                                                              homepage_id_wise)

        season_df, current_season_name = SeasonalModel.get_specific_seasons(history_content_df)

        Logging.info("Calculating Seasonal Score from " + user_label + " UBD data")
        season_df_view_count = season_df.groupby([CONTENT_ID]).size().to_frame(VIEW_COUNT)
        season_df_total_duration_content = season_df.groupby([CONTENT_ID]).sum()
        season_df_total_duration_content = season_df_total_duration_content[[DURATION]]
        final_recommendation_df = pd.merge(season_df_total_duration_content, season_df_view_count,
                                           left_index=True, right_index=True).sort_values(by=[DURATION, VIEW_COUNT],
                                                                                          ascending=False)
        scaled_features_df = MinMaxScaler().fit_transform(final_recommendation_df.values)
        scaled_features_df = pd.DataFrame(scaled_features_df, index=final_recommendation_df.index,
                                          columns=final_recommendation_df.columns).rename(
            columns={DURATION: SCALED_DURATION, VIEW_COUNT: SCALED_VIEW_COUNT}
        )
        final_recommendation_df = final_recommendation_df.merge(scaled_features_df, how=INNER,
                                                                left_index=True, right_index=True)
        final_recommendation_df[SCORE] = round((final_recommendation_df[SCALED_DURATION] * 0.7) +
                                               (final_recommendation_df[SCALED_VIEW_COUNT] * 0.3), 3)
        Logging.info("Preparing Seasonal Model Dataframe Output")
        final_recommendation_df = final_recommendation_df.sort_values(by=SCORE, ascending=False)
        final_recommendation_df[CREATED_ON] = datetime.utcnow().isoformat()
        seasonal_content_df = final_recommendation_df[[SCORE, CREATED_ON]].rename_axis(CONTENT_ID)\
            .reset_index()
        seasonal_content_df[SEASON] = current_season_name
        seasonal_content_df[REC_TYPE] = SEASONAL
        if homepage_id_wise:
            seasonal_content_df = pd.merge(seasonal_content_df, content_homepage_id_mapping, on=CONTENT_ID, how=LEFT)
            seasonal_content_df = seasonal_content_df[~seasonal_content_df[HOMEPAGE_ID].isnull()].reset_index(drop=True)
            seasonal_content_df = seasonal_content_df.explode(column=HOMEPAGE_ID)
            seasonal_content_df = SeasonalModel.filter_by_dedicated_homepage_id(seasonal_content_df, user_label,
                                                                                filter_on_ubd=False)
            seasonal_content_df = SeasonalModel.filter_by_relevant_homepage_id(seasonal_content_df, current_season_name,
                                                                               user_label)
            seasonal_content_df = seasonal_content_df.sort_values(by=[HOMEPAGE_ID, SCORE], ascending=[True, False])
        else:
            seasonal_content_df = pd.merge(seasonal_content_df, content_homepage_id_mapping, on=CONTENT_ID, how=LEFT)
            seasonal_content_df = seasonal_content_df[~seasonal_content_df[HOMEPAGE_ID].isnull()].reset_index(drop=True)
            seasonal_content_df[HOMEPAGE_ID] = seasonal_content_df.apply(lambda item: item[HOMEPAGE_ID][0], axis=1)
            seasonal_content_df = SeasonalModel.filter_by_dedicated_homepage_id(seasonal_content_df, user_label,
                                                                                filter_on_ubd=False)
            seasonal_content_df = SeasonalModel.filter_by_relevant_homepage_id(seasonal_content_df, current_season_name,
                                                                               user_label)
            seasonal_content_df = seasonal_content_df.sort_values(by=SCORE, ascending=False)
        return seasonal_content_df

    @staticmethod
    def get_seasonal_content_all_content(graph):
        SeasonalModel.get_data()
        list_of_key_prefix = []
        homepage_id_wise = False
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_df = []
        Logging.info("Get Seasonal Content based on All Content")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating seasonal content based on All Content for " + user_label + " users")
            output_df = SeasonalModel.seasonal_algorithm(graph, user_label, homepage_id_wise)
            Logging.info("Success Preparing Seasonal Dataframe Output")
            key_prefix = SERVICE_NAME + ":" + SEASONAL_MODULE_NAME + ":" + user_label + ":" + ALL_CONTENT_BASED
            list_of_key_prefix.append(key_prefix)
            list_output_df.append(output_df)
        seasonal_pay_tv_all_content, seasonal_no_pay_tv_all_content = list_output_df

        Logging.info("Preparing the output to JSON Schema")
        # Convert dataframe output to json output
        seasonal_pay_tv_all_content_dict = SeasonalModel.get_dict_format_output(seasonal_pay_tv_all_content,
                                                                                list_of_key_prefix[0],
                                                                                homepage_id_wise)
        seasonal_no_pay_tv_all_content_dict = SeasonalModel.get_dict_format_output(seasonal_no_pay_tv_all_content,
                                                                                   list_of_key_prefix[1],
                                                                                   homepage_id_wise)
        end = time.time()
        duration = end - start
        Logging.info("Duration for Get Seasonal Content based on All Content is " + str(duration) + " seconds")
        SeasonalModel.clear_data()
        return seasonal_pay_tv_all_content_dict, seasonal_no_pay_tv_all_content_dict

    @staticmethod
    def get_seasonal_content_homepage_id(graph):
        SeasonalModel.clear_data()
        SeasonalModel.get_data()
        list_of_key_prefix = []
        homepage_id_wise = True
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_df = []
        Logging.info("Get Seasonal Content based on Homepage_id")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating seasonal content based on Homepage_id for " + user_label + " users")
            output_df = SeasonalModel.seasonal_algorithm(graph, user_label, homepage_id_wise)
            Logging.info("Success Preparing Seasonal Dataframe Output")
            key_prefix = SERVICE_NAME + ":" + SEASONAL_MODULE_NAME + ":" + user_label + ":" + HOMEPAGE_ID_BASED
            list_of_key_prefix.append(key_prefix)
            list_output_df.append(output_df)
        seasonal_pay_tv_homepage_id, seasonal_no_pay_tv_homepage_id = list_output_df

        Logging.info("Preparing the output to JSON Schema")
        # Convert dataframe output to json output
        seasonal_pay_tv_homepage_id_dict = SeasonalModel.get_dict_format_output(seasonal_pay_tv_homepage_id,
                                                                                list_of_key_prefix[0],
                                                                                homepage_id_wise)
        seasonal_no_pay_tv_homepage_id_dict = SeasonalModel.get_dict_format_output(seasonal_no_pay_tv_homepage_id,
                                                                                   list_of_key_prefix[1],
                                                                                   homepage_id_wise)

        end = time.time()
        duration = end - start
        Logging.info("Duration for Get Seasonal Content based on Homepage_id is " + str(duration) + " seconds")
        return seasonal_pay_tv_homepage_id_dict, seasonal_no_pay_tv_homepage_id_dict
