from datetime import datetime
import time
import pandas as pd

from offline_results.common.config import CONFIG_HOMEPAGE_PAYTV, CONFIG_HOMEPAGE_NO_PAYTV
from offline_results.common.constants import NO_PAY_TV, PAY_TV, CONTENT_ID, \
    TV_CHANNEL_LIST, SERVICE_NAME, ALL_CONTENT_BASED, RECORDS, TRENDING_MODULE_DEFAULT, \
    HOMEPAGE_ID, CREATED_ON, VIEW_COUNT, SUM, CONTENT_STATUS, LEFT, ACTIVE, IS_PAY_TV, INDEX, \
    MODEL_NAME, HOMEPAGE_ID_BASED, TRENDING, INNER, REC_TYPE, TRENDING_FALLBACK
from offline_results.recommendation.homepage_recommendation.trending.trending_model import TrendingModel
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.utils.logger import Logging


class TrendingFallback:

    viewed_relation_history_df = pd.DataFrame()

    @staticmethod
    def get_data():
        obj = TrendingFallback
        if len(obj.viewed_relation_history_df) != 0: return
        obj.viewed_relation_history_df = RecommendationUtils.user_viewed_data_from_s3()

    @staticmethod
    def clear_data():
        obj = TrendingFallback
        obj.viewed_relation_history_df = pd.DataFrame()

    @staticmethod
    def default_trending_homepage_algorithm(user_label, graph):

        df = TrendingFallback.viewed_relation_history_df
        if user_label == PAY_TV:
            df_paytv = df.loc[lambda df: (df[IS_PAY_TV] == True) & (df[CONTENT_STATUS] == ACTIVE)]
            try:
                Logging.info("Fetch content-homepage_id mapping")
                paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=PAY_TV,
                                                                                  homepage_id_wise=True)
            except Exception:
                graph = ANGraphDb.new_connection_config().graph
                Logging.info("Re-trying to Fetch content-homepage_id mapping")
                paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=PAY_TV,
                                                                                  homepage_id_wise=True)
            df_active_homepage = paytv_homepage_df.explode(column=HOMEPAGE_ID)
            df_paytv_label_merge_with_homepage_id = pd.merge(df_paytv, df_active_homepage, on=CONTENT_ID,
                                                             how=LEFT)
            df_paytv_label_merge_with_homepage_id = df_paytv_label_merge_with_homepage_id[~df_paytv_label_merge_with_homepage_id[HOMEPAGE_ID].isna()].reset_index(drop=True)
            df_trending = pd.DataFrame.from_dict(CONFIG_HOMEPAGE_PAYTV, orient=INDEX).reset_index()
            df_trending.rename(columns={INDEX: HOMEPAGE_ID, 0: MODEL_NAME}, inplace=True)
            df_trending = df_trending[df_trending[MODEL_NAME] == TRENDING]
            df_merge_trending = pd.merge(df_trending, df_paytv_label_merge_with_homepage_id, on=HOMEPAGE_ID,
                                         how=INNER)
            df_view_count = df_merge_trending[~df_merge_trending[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        else:
            df_no_paytv = df.loc[lambda df: (df[IS_PAY_TV] == False) & (df[CONTENT_STATUS] == ACTIVE)]
            try:
                Logging.info("Fetch content-homepage_id mapping")
                no_paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=NO_PAY_TV,
                                                                                     homepage_id_wise=True)
            except Exception:
                graph = ANGraphDb.new_connection_config().graph
                Logging.info("Re-trying to Fetch content-homepage_id mapping")
                no_paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=NO_PAY_TV,
                                                                                     homepage_id_wise=True)
            no_paytv_homepage_df = no_paytv_homepage_df.explode(column=HOMEPAGE_ID)
            df_no_paytv_label_merge_with_homepage_id = pd.merge(df_no_paytv, no_paytv_homepage_df, on=CONTENT_ID,
                                                                how=LEFT)
            df_no_paytv_trending = pd.DataFrame.from_dict(CONFIG_HOMEPAGE_NO_PAYTV, orient=INDEX).reset_index()
            df_no_paytv_trending.rename(columns={INDEX: HOMEPAGE_ID, 0: MODEL_NAME}, inplace=True)
            df_no_paytv_trend = df_no_paytv_trending[df_no_paytv_trending[MODEL_NAME] == TRENDING]
            df_no_paytv_filter_homepage_id = pd.merge(df_no_paytv_trend, df_no_paytv_label_merge_with_homepage_id,
                                                      on=HOMEPAGE_ID,
                                                      how=INNER)
            df_view_count = df_no_paytv_filter_homepage_id[
                ~df_no_paytv_filter_homepage_id[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        unique_homepage_list = df_view_count[HOMEPAGE_ID]
        unique_homepage_list = unique_homepage_list.unique()
        key_prefix = SERVICE_NAME + ":" + TRENDING_MODULE_DEFAULT + ":" + user_label + ":" + HOMEPAGE_ID_BASED
        homepage_based_rec = {}
        for homepage_id in unique_homepage_list:
            key_prefix_cls = key_prefix + ":" + str(homepage_id)
            filter_df = df_view_count[df_view_count[HOMEPAGE_ID] == homepage_id]
            filter_df = filter_df.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
            filter_df = filter_df.sort_values(by=VIEW_COUNT, ascending=False)
            filter_df.drop(columns=VIEW_COUNT, axis=1, inplace=True)
            filter_df = filter_df.head(10)
            filter_df[CREATED_ON] = datetime.utcnow().isoformat()
            filter_df[REC_TYPE] = TRENDING_FALLBACK
            result = filter_df.to_dict(orient=RECORDS)
            homepage_based_rec[key_prefix_cls] = result
        return homepage_based_rec

    @staticmethod
    def default_trending_all_content_algorithm(user_label, graph):

        df = TrendingFallback.viewed_relation_history_df
        if user_label == PAY_TV:
            df_paytv = df.loc[lambda df: (df[IS_PAY_TV] == True) & (df[CONTENT_STATUS] == ACTIVE)]
            try:
                Logging.info("Fetch content-homepage_id mapping")
                paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=PAY_TV,
                                                                                  homepage_id_wise=False)
            except Exception:
                graph = ANGraphDb.new_connection_config().graph
                Logging.info("Re-trying to Fetch content-homepage_id mapping")
                paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=PAY_TV,
                                                                                  homepage_id_wise=False)
            df2_new = paytv_homepage_df.explode(column=HOMEPAGE_ID)
            df_paytv_label_merge_with_homepage_id = pd.merge(df_paytv, df2_new, on=CONTENT_ID,
                                                             how=LEFT)
            df_trending = pd.DataFrame.from_dict(CONFIG_HOMEPAGE_PAYTV, orient=INDEX).reset_index()
            df_trending.rename(columns={INDEX: HOMEPAGE_ID, 0: MODEL_NAME}, inplace=True)
            df_trend = df_trending[df_trending[MODEL_NAME] == TRENDING]
            df_filter_homepage_id = pd.merge(df_paytv_label_merge_with_homepage_id, df_trend, on=HOMEPAGE_ID, how=INNER)
            df_filter_content = df_filter_homepage_id[~df_filter_homepage_id[CONTENT_ID].isin(TV_CHANNEL_LIST)]
            df_view_sum = df_filter_content.groupby([CONTENT_ID], as_index=False)[VIEW_COUNT].agg(SUM)
            df_homepage_id = df_filter_content[[HOMEPAGE_ID, CONTENT_ID]]
            df_homepage_id = pd.merge(df_view_sum, df_homepage_id, on=CONTENT_ID,
                                      how=LEFT)
        else:
            df_no_paytv = df.loc[lambda df: (df[IS_PAY_TV] == False) & (df[CONTENT_STATUS] == ACTIVE)]
            try:
                Logging.info("Fetch content-homepage_id mapping")
                no_paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=NO_PAY_TV,
                                                                                     homepage_id_wise=False)
            except Exception:
                graph = ANGraphDb.new_connection_config().graph
                Logging.info("Re-trying to Fetch content-homepage_id mapping")
                no_paytv_homepage_df = TrendingModel.get_content_homepage_id_mapping(graph, user_label=NO_PAY_TV,
                                                                                     homepage_id_wise=False)
            df_no_paytv_homepage = no_paytv_homepage_df.explode(column=HOMEPAGE_ID)
            df_no_paytv_label_merge_with_homepage_id = pd.merge(df_no_paytv, df_no_paytv_homepage, on=CONTENT_ID,
                                                                how=LEFT)
            df_no_paytv_trending = pd.DataFrame.from_dict(CONFIG_HOMEPAGE_NO_PAYTV, orient=INDEX).reset_index()
            df_no_paytv_trending.rename(columns={INDEX: HOMEPAGE_ID, 0: MODEL_NAME}, inplace=True)
            df_no_paytv_trend = df_no_paytv_trending[df_no_paytv_trending[MODEL_NAME] == TRENDING]
            df_no_paytv_filter_homepage_id = pd.merge(df_no_paytv_label_merge_with_homepage_id, df_no_paytv_trend,
                                                      on=HOMEPAGE_ID,
                                                      how=INNER)
            df_no_paytv_filter_content = df_no_paytv_filter_homepage_id[
                ~df_no_paytv_filter_homepage_id[CONTENT_ID].isin(TV_CHANNEL_LIST)]
            df_view_sum = df_no_paytv_filter_content.groupby([CONTENT_ID], as_index=False)[VIEW_COUNT].agg(SUM)
            df_homepage_id = df_no_paytv_filter_content[[HOMEPAGE_ID, CONTENT_ID]]
            df_homepage_id = pd.merge(df_view_sum, df_homepage_id, on=CONTENT_ID,
                                      how=LEFT)
        df_homepage_id = df_homepage_id.drop_duplicates(subset=[CONTENT_ID], keep='first')
        filter_df = df_homepage_id.sort_values(by=VIEW_COUNT, ascending=False).iloc[:20, :]
        filter_df.drop(columns=VIEW_COUNT, axis=1, inplace=True)
        filter_df.drop_duplicates(CONTENT_ID, inplace=True)
        filter_df[CREATED_ON] = datetime.utcnow().isoformat()
        filter_df[REC_TYPE] = TRENDING_FALLBACK
        all_content_based_rec = {}
        key_prefix = SERVICE_NAME + ":" + TRENDING_MODULE_DEFAULT + ":" + user_label + ":" + ALL_CONTENT_BASED
        result = filter_df.to_dict(orient=RECORDS)
        all_content_based_rec[key_prefix] = result
        return all_content_based_rec

    @staticmethod
    def get_default_trending_homepage_id(graph):
        TrendingFallback.clear_data()
        TrendingFallback.get_data()
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_dict = []
        Logging.info("Get Fallback Trending Content based on Homepage_id")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating fallback trending content based on Homepage_id for " + user_label + " users")
            output_dict = TrendingFallback.default_trending_homepage_algorithm(user_label, graph)
            Logging.info("Success Preparing Fallback Trending Dataframe Output")
            list_output_dict.append(output_dict)
        fallback_trending_pay_tv_homepage_id_dict, fallback_trending_no_pay_tv_homepage_id_dict = list_output_dict
        end = time.time()
        duration = end-start
        Logging.info("Duration for Get Fallback Trending Content based on Homepage_id is " + str(duration) + " seconds")
        return fallback_trending_pay_tv_homepage_id_dict, fallback_trending_no_pay_tv_homepage_id_dict

    @staticmethod
    def get_default_trending_all_content(graph):
        TrendingFallback.get_data()
        list_user_label = [PAY_TV, NO_PAY_TV]
        list_output_dict = []
        Logging.info("Get Fallback Trending Content based on All Content")
        start = time.time()
        for user_label in list_user_label:
            Logging.info("Start generating fallback trending content based on All Content for " + user_label + " users")
            output_dict = TrendingFallback.default_trending_all_content_algorithm(user_label, graph)
            Logging.info("Success Preparing Fallback Trending Dataframe Output")
            list_output_dict.append(output_dict)
        fallback_trending_pay_tv_all_content_dict, fallback_trending_no_pay_tv_all_content_dict = list_output_dict
        end = time.time()
        duration = end - start
        Logging.info("Duration for Get Fallback Trending Content based on All Content is " + str(duration) + " seconds")
        TrendingFallback.clear_data()
        return fallback_trending_pay_tv_all_content_dict, fallback_trending_no_pay_tv_all_content_dict
